@@ -23,9 +23,12 @@ install("matplotlib")
 install("tweepy")
 install("nltk")
 install("textblob")
+import("fastdtw")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
 import ast
+from fastdtw import fastdtw
+from scipy.spatial.distance import euclidean
 import pandas as pd
 import numpy as np
 import urllib3
@@ -37,6 +40,7 @@ import matplotlib.pyplot as plt
 import tweepy
 from os import path
 from datetime import timedelta
+from datetime import datetime
 
 #---------------------------------------------NLP-Part-------------------------------------------------------------------------------#
 from textblob import TextBlob
@@ -502,94 +506,95 @@ def graph_features(DiG,df):
     return output
 
 def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret, iter_tw,save_by, fichero):
-	profile_iter = save_by
-	k=0
-	auth = tweepy.OAuthHandler(consumerKey[0], consumerSecret[0])
-	auth.set_access_token(accessToken[0], accessTokenSecret[0])
-	api = tweepy.API(auth)
-	nc= 0
-	ultima = 0
-	origen1 = time.time()
-	while ((k<= (len(node_list)+profile_iter)) & (ultima !=2) ):
-		tweets_user=[]
-		list_users_noInfo = []
-		origen2 = time.time()
-		i=0
-		if ((len(node_list)>= k) & ((len(node_list)-profile_iter) <=k)):
-			node_list1 = node_list[k:len(node_list)]
-			ultima = 1
-		else:
-			node_list1 = node_list[k:k+profile_iter]
-		while(i <(len(node_list1))):         
-			pri = 0
-			itera=0
-			max_id=""
-			intentos = 0
-			while itera != iter_tw:
-				try:
-					if pri ==0:
-						user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended")      
-						tweets_user.append(user)
-						max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
-						pri=1
-						itera = itera+1
-					else:
-						user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended", max_id=max_id)
-						tweets_user.append(user)
-						max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
-						itera = itera+1
-						intentos=0 # Para si justo es el primero el que no se puede extraer y no es problema de agotar claves
-				except:
-					nc = nc+1
-					n_clave = len(consumerKey)
-					if(nc == n_clave):
-						nc= 0
-					auth = tweepy.OAuthHandler(consumerKey[nc], consumerSecret[nc])
-					auth.set_access_token(accessToken[nc], accessTokenSecret[nc])
-					api = tweepy.API(auth)
-					intentos = intentos + 1
-					if(intentos == 2):         
-						list_users_noInfo.append(node_list1[i])
-						intentos = 0
-						itera=iter_tw
-			i = i+1       
-		destino2 = time.time()
-		aux=[]
-		for i in range( len(tweets_user)):
-			for j in range(len(tweets_user[i])):          
-				aux.append(tweets_user[i][j]._json)     
-		if(len(aux) != 0):  
-			df = pd.DataFrame(aux)
-			df = df.rename(columns = {"created_at": "created_at_tw"})
-			df= df.rename(columns = {"lang": "lang_tw"})
-			df = pd.concat([df.drop(['user'], axis=1), df['user'].apply(pd.Series)], axis=1)
-			df = df.rename(columns={"followers_count": "followers",
-							  "friends_count": "followees",
-							  "retweet_count": "retweets",
-							  "favorite_count": "favorites",
-							  "favourites_count": "favorites_count",
-							  "full_text": "text"})
-			df2 = metadata_extraction(df)
-			if path.exists("/content/"+ fichero):
-				df2.to_csv(fichero,mode="a", index=False, header=False)
-			else:
-				df2.to_csv(fichero,index=False)         
-			if len(list_users_noInfo) !=0:
-				nodes = pd.DataFrame()
-				nodes["nodos"] = list_users_noInfo
-				if path.exists("/content/node_without_"+fichero):
-					nodes.to_csv("node_without_"+ fichero, mode ="a",index=False, header=False)          
-				else:
-					nodes.to_csv("node_without_"+ fichero, index=False)
-			k = k+profile_iter        
-			if ultima ==1:          
-				ultima =2
-		else:
-			print("Repetimos")
-	print("Lista de usuarios sin metadata")
-	print(list_users_noInfo)
-	destino1 = time.time()
-	return df2, list_users_noInfo
+    # get_user Returns the 20 most recent statuses posted from the authenticating user or the user specified. 
+    profile_iter = save_by
+    k=0
+    auth = tweepy.OAuthHandler(consumerKey[0], consumerSecret[0])
+    auth.set_access_token(accessToken[0], accessTokenSecret[0])
+    api = tweepy.API(auth)
+    nc= 0
+    ultima = 0
+    origen1 = time.time()
+    while ((k<= (len(node_list)+profile_iter)) & (ultima !=2) ):
+      tweets_user=[]
+      list_users_noInfo = []
+      origen2 = time.time()
+      i=0
+      if ((len(node_list)>= k) & ((len(node_list)-profile_iter) <=k)):
+        node_list1 = node_list[k:len(node_list)]
+        ultima = 1
+      else:
+        node_list1 = node_list[k:k+profile_iter]
+      while(i <(len(node_list1))):         
+        pri = 0
+        itera=0
+        max_id=""
+        intentos = 0
+        while itera != iter_tw:
+          try:
+            if pri ==0:
+              user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended")      
+              tweets_user.append(user)
+              max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
+              pri=1
+              itera = itera+1
+            else:
+              user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended", max_id=max_id)
+              tweets_user.append(user)
+              max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
+              itera = itera+1
+              intentos=0 # Para si justo es el primero el que no se puede extraer y no es problema de agotar claves
+          except:
+            nc = nc+1
+            n_clave = len(consumerKey)
+            if(nc == n_clave):
+              nc= 0
+            auth = tweepy.OAuthHandler(consumerKey[nc], consumerSecret[nc])
+            auth.set_access_token(accessToken[nc], accessTokenSecret[nc])
+            api = tweepy.API(auth)
+            intentos = intentos + 1
+            if(intentos == 2):         
+              list_users_noInfo.append(node_list1[i])
+              intentos = 0
+              itera=iter_tw
+        i = i+1       
+      destino2 = time.time()
+      aux=[]
+      for i in range( len(tweets_user)):
+        for j in range(len(tweets_user[i])):          
+            aux.append(tweets_user[i][j]._json)     
+      if(len(aux) != 0):  
+        df = pd.DataFrame(aux)
+        df = df.rename(columns = {"created_at": "created_at_tw"})
+        df= df.rename(columns = {"lang": "lang_tw"})
+        df = pd.concat([df.drop(['user'], axis=1), df['user'].apply(pd.Series)], axis=1)
+        df = df.rename(columns={"followers_count": "followers",
+                              "friends_count": "followees",
+                              "retweet_count": "retweets",
+                              "favorite_count": "favorites",
+                              "favourites_count": "favorites_count",
+                              "full_text": "text"})
+        df2 = metadata_extraction(df)
+        if path.exists("/content/"+ fichero):
+          df2.to_csv(fichero,mode="a", index=False, header=False)
+        else:
+          df2.to_csv(fichero,index=False)         
+        if len(list_users_noInfo) !=0:
+          nodes = pd.DataFrame()
+          nodes["nodos"] = list_users_noInfo
+          if path.exists("/content/node_without_"+fichero):
+            nodes.to_csv("node_without_"+ fichero, mode ="a",index=False, header=False)          
+          else:
+             nodes.to_csv("node_without_"+ fichero, index=False)
+        k = k+profile_iter        
+        if ultima ==1:          
+          ultima =2
+      else:
+        print("Repetimos")
+    print("Lista de usuarios sin metadata")
+    print(list_users_noInfo)
+    destino1 = time.time()
+    return df2, list_users_noInfo
 
 
 #----------------------------------Extract metadata-----------------------------------------------------------#
@@ -1037,19 +1042,21 @@ def filter_possible_irregular_profiles(data_join):
     return data_join
 
 def NLP_analysis(data, keyword_corpus, hashtag_corpus):
-    tweets = data["text_tweets_days_list"]
-    tweets_join = [j for i in tweets for j in i]
-    tweets_df = pd.DataFrame(tweets_join)
+    tweets_df = pd.DataFrame({'text':data["text_tweets_days_list"],'RT':data["RT_days_list"],'lang': data["lang_days_list"]})
+    tweets_dff = pd.DataFrame()
+    for i in range(len(tweets_df)):
+      aux = pd.DataFrame({'text':tweets_df["text"].iloc[i],'RT':tweets_df["RT"].iloc[i],'lang': tweets_df["lang"].iloc[i]})
+      tweets_dff = tweets_dff.append(aux, ignore_index=False).reset_index(drop=True)
     tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
     tweets_df = sentiental_analysis_features(tweets_df)
     sample_data_tokenized = tokenize_tweets(tweets_df)
     tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
     tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
     tweets_df["terrorist_suggestion"] = 0
-    print(data["screen_name"])
     tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
     tweets_irr_words = len(tweets_df[tweets_df["terrorist_suggestion"]==1])
     if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
+      data["per_hash_key"] = 0
       data["negative_opinion"] = 0
       data["negative_fact"] = 0
       data["neutral"] = 0
@@ -1057,11 +1064,11 @@ def NLP_analysis(data, keyword_corpus, hashtag_corpus):
       data["positive_opinion"] = 0
     else:
       data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]>0.6)]= 2
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["subjectivity"]>0.2) & (tweets_df["polarity"] > -0.1) & (tweets_df["subjectivity"]<0.4)]= 4
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] > 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] > 0.1) & (tweets_df["subjectivity"]>0.6)]= 6
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]>=0.6)]= 2
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["polarity"] > -0.1)]= 4
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]>=0.6)]= 6
       if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
         data["negative_opinion"] = 0
       else:
@@ -1082,6 +1089,7 @@ def NLP_analysis(data, keyword_corpus, hashtag_corpus):
         data["positive_opinion"] = 0
       else:
         data["positive_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==6]) / tweets_irr_words)
+        
       
     return data,tweets_df
 	
@@ -1120,7 +1128,7 @@ def text_to_list_features_conversor (data):
         for i in range(len(text_days)):
             #print(i)
             tweet_date_day = tweet_date_days[i].replace("), ",")??").strip("[]").split("??")
-            text_tweets_day = data["text_tweets_days_list"].iloc[index][i].replace("'}, {'","'}??{'").strip("[]").split("??")
+            text_tweets_day = data["text_tweets_days_list"].iloc[index][i].replace("', '","??").strip("[]'").split("??")
             if len(RT_days[i])==1:
                  RT_day = list(RT_days[i])
             else:
@@ -1148,7 +1156,6 @@ def text_to_list_features_conversor (data):
         data["lang_days_list"].iloc[index] = list_lang_days
     return data
 
-
 	
 def evaluate_NLP_profiles(data,keyword_corpus, hashtag_corpus):	
 	final_df= pd.DataFrame()
